#include <ubb/ubb.h>
#include <stdio.h>
#include <time.h>

/* Pin assignments:
 *
 *           M74HC595B1R
 *
 *           QB |----| VCC   UBB_VDD
 *           QC |    | QA
 *           QD |    | SI    UBB_DAT2
 *           QE |    | G     UBB_DAT0
 *           QF |    | RCK   UBB_DAT1
 *           QG |    | SCK   UBB_CLK
 *           QH |    | SCLR  UBB_CMD
 * UBB_GND  GND |----| QH'   UBB_DAT3
 */

#define SHIFT_SI    UBB_DAT2
#define SHIFT_QH    UBB_DAT3
#define SHIFT_SCLR  UBB_CMD
#define SHIFT_SCK   UBB_CLK
#define SHIFT_G     UBB_DAT0
#define SHIFT_RCK   UBB_DAT1

struct timespec tSCKs   = {0, 15},
                tWH     = {0, 15},
                tWL     = {0, 15};

void shift_in(uint8_t v)
{
    uint8_t mask;

    for (mask = 0x80; mask; mask >>= 1)
    {
        if (v & mask)
        {
            printf("1");
            SET(SHIFT_SI);
        }
        else
        {
            printf("0");
            CLR(SHIFT_SI);
        }

        nanosleep(&tSCKs, NULL);
        SET(SHIFT_SCK);
        nanosleep(&tWH, NULL);
        CLR(SHIFT_SCK);
    }

    printf("\n");
}

uint8_t shift_out()
{
    uint8_t mask, result = 0;

    for (mask = 0x80; mask; mask >>= 1)
    {
        if (PIN(SHIFT_QH))
        {
            result |= mask;
            printf("1");
        }
        else
            printf("0");

        nanosleep(&tSCKs, NULL);
        SET(SHIFT_SCK);
        nanosleep(&tWH, NULL);
        CLR(SHIFT_SCK);
    }

    printf("\n");
    return result;
}

int main(int argc, char *argv[])
{
    uint8_t result = 0;

    if (ubb_open(0) < 0) {
        perror("ubb_open");
        return 1;
    }

    ubb_power(1);
    printf("Power on.\n");

    OUT(SHIFT_SI);
    OUT(SHIFT_RCK);
    OUT(SHIFT_SCLR);
    OUT(SHIFT_SCK);
    OUT(SHIFT_G);
    IN(SHIFT_QH);

    CLR(SHIFT_SCK);
    CLR(SHIFT_RCK);

    /* Clear the register. */

    CLR(SHIFT_SCLR);
    SET(SHIFT_SCK);
    nanosleep(&tWL, NULL);
    CLR(SHIFT_SCK);
    SET(SHIFT_SCLR);

    /* Shift in a value. */

    shift_in(0xa6);

    /* Shift out the value. */

    result = shift_out();

    printf("Result: %x\n", result);

    printf("Closing...\n");
    ubb_close(0);

    return 0;
}
